/**
 * Memory Analyser  version 1.00
 * Copyright (c) 2018, Tycho Veltmeijer.
 * All rights reserved.
**/

#include "Filter_Dump.h"

/*Description flags from Protect member
source http://msdn.microsoft.com/en-us/library/windows/desktop/aa366786(v=vs.85).aspx
*/
char* pageNames[32] = {"Disabled", //0x1
	"Readable", //0x2
	"Read and writeable", //0x4
	"Read and writeable", //0x8
	"Executeable (code)", //0x10
	"Executeable and readable (code)", //0x20
	"Read, write and executeable", //0x40
	"Read, write and executeable", //0x80
	"PAGE_GUARD", //0x100
	"PAGE_NOCACHE", //0x200
	"PAGE_WRITECOMBINE", //0x400
	"","","","","","","","","","","","","","","","","","","","",""};

/*Call CreateMemoryDump and write header*/
DUMP_SESSION* CreateDumpIndexMemoryDump(char* outputDir, char* outputFile, char* optionalOutputFile, int flags) {
	int writtenBytes=0;
	DUMP_SESSION* newdump;

	newdump = CreateMemoryDump(outputDir, outputFile, optionalOutputFile, flags);
	if(optionalOutputFile!=0)
		WriteFile(newdump->optionalDumpFile,"Module              Dump Address  Virtual Address	Size\r\n", strlen("Module              Dump Address  Virtual Address	Size\r\n"), (LPDWORD) &writtenBytes, NULL);

	return newdump;
}

DUMP_SESSION* InitiateDumpSmall(char* location, char* argument, int analyse_type) {
	return CreateDumpIndexMemoryDump(location, "small_dump.txt", "small_dump_index.txt", DS_WRITE_LOCATION_STRING);
}

DUMP_SESSION* InitiateDumpBig(char* location, char* argument, int analyse_type) {
	return CreateMemoryDump(location, "big_dump.txt", 0, DS_WRITE_UNREADABLE_DATA_TO_DUMP);
}

void DestroyDump(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

//This method write complete memory dump
void WriteMemoryDump(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	char* moveBuff=buff;
	int writtenBytes;
	memlong curPos=0;
	int i;

	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {

		/*Not all memory is used by the analyed process
		Memory with flag DS_WRITE_UNREADABLE_DATA_TO_DUMP is filled with '0' bytes*/
		if(dmp->unlocated_size!=0 && dmp->flags&DS_WRITE_UNREADABLE_DATA_TO_DUMP) {
			while(dmp->unlocated_size!=0) {
				WriteFile(dmp->dumpFile, dmp->zerobuf, (dmp->unlocated_size<1000 ? dmp->unlocated_size : 1000), (LPDWORD) &writtenBytes, NULL);
				dmp->unlocated_size -= writtenBytes;
			}
			dmp->unlocated_size = 0;
		}
		
		/*Write virtual pointer to the other file  
		*/
		if(dmp->flags&DS_WRITE_LOCATION_STRING) {
			GetFileSizeEx(dmp->dumpFile, (PLARGE_INTEGER) &curPos); 

			if(dmp->optionalDumpFile!=0 && process !=0) {
				/*Try to figure out from which module this peace of memory is*/
				if(GetModuleFileNameEx(process, (HMODULE) memInfo->AllocationBase, dmp->string_location, 1000)!=0
					|| GetModuleFileNameEx(process, (HMODULE) memInfo->BaseAddress, dmp->string_location, 1000)!=0) {
					char* fileName=dmp->string_location;
					fileName += strlen(fileName);

					while(*fileName!='\\' && *fileName!='/')
						fileName--;

					fileName++;

					WriteFile(dmp->optionalDumpFile, fileName, strlen(fileName), (LPDWORD) &writtenBytes, NULL);

					if(strlen(fileName)<20)
						for(i=0; i<20-strlen(fileName); i++)
							WriteFile(dmp->optionalDumpFile, " ", strlen(" "), (LPDWORD) &writtenBytes, NULL);
				} else {
					WriteFile(dmp->optionalDumpFile, "                    ", 20, (LPDWORD) &writtenBytes, NULL);
				}

				sprintf(dmp->string_location, "0x%p	  0x%p-0x%p	%d	"
					,curPos //Position in file
					,memInfo->BaseAddress //Virtual memory position
					,(SIZE_T)memInfo->BaseAddress + memInfo->RegionSize 
					,memInfo->RegionSize 
					);

				flagsToStr(memInfo->Protect, pageNames, dmp->string_location); //flag to readable string

				WriteFile(dmp->optionalDumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);

				WriteFile(dmp->optionalDumpFile, "\r\n", strlen("\r\n"), (LPDWORD) &writtenBytes, NULL);
			} else {
				sprintf(dmp->string_location, "\n\n[Base address: %d, Region: %d]\n", memInfo->BaseAddress, memInfo->RegionSize);
				WriteFile(dmp->dumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);
			}
		}

		WriteFile(dmp->dumpFile, buff, memInfo->RegionSize, (LPDWORD) &writtenBytes, NULL);
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}
}

/*Flag integer to readable string*/
void flagsToStr(int inFlags, char* inStr[], char*out) {
	int currentFlag=1;
	int countFlag=0;
	int firstFlagFound=0;

	while(inFlags!=0) {
		if(inFlags&currentFlag 
		&& *inStr[countFlag]!=0) {  
			if(firstFlagFound) 
				strcat(out, ", ");

			strcat(out, inStr[countFlag]); 

			firstFlagFound|=1; 
			inFlags^=currentFlag; 
		}
		currentFlag*=2; 
		countFlag++;
	}
}