/**
 * Memory Analyser  version 1.00
 * Copyright (c) 2018, Tycho Veltmeijer.
 * All rights reserved.
**/

#include "Filter_General.h"

/*--------------------------------------------------------Filter ASCI--------------------------------------------------------*/
DUMP_SESSION* InitiateFilterGeneralASCII(char* location, char* argument, int analyse_type) {
	return CreateMemoryDump(location, "General_URL_ASCII_Search.txt", 0, 
		(argument==(char*) -1 ? DS_WRITE_LOCATION_STRING : strtol(argument, NULL, 10))
	);
}

/*Search for (ASCII) URL's in the memory (starting with http://)*/
void ApplyFilterGeneralASCII(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	char* moveBuff=buff;
	char* moveBuff2=0;
	int writtenBytes;
	int readSize=0;
	int isOkayURL=0;


	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {
		for(; moveBuff<(buff + memInfo->RegionSize); moveBuff++) {
			//Is it possible a URL?
			if(memcmp(moveBuff, "http://",  7) == 0) {
				
				moveBuff2 = moveBuff;
				moveBuff2 += 7; //skip 'http://'

				isOkayURL = 1;

				/*Check if the domain meets the requirments:
				- Letters a - z
				- numbers 0-9
				- '-' or '.' */
				while(*moveBuff2 != 0) {
					if((*moveBuff2 < 'a' || *moveBuff2 > 'z') 
						&& (*moveBuff2 < '0' || *moveBuff2 > '9') 
						&& *moveBuff2 != '-'
						&& *moveBuff2 != '.') {
							if(*moveBuff2 == '/') //domain stops, path begins
								break;
							isOkayURL = 0;
							break;
					}
					moveBuff2++;
				}

				/*Check if the URL meets the requirments:
				- No space (this is a %20)
				- no newline (0D of 0A)
				*/
				while(*moveBuff2 != 0) {
					if(*moveBuff2 == ' ' || *moveBuff2 == 0x0D || *moveBuff2 == 0x0A) {
						isOkayURL = 0;
						break;
					}
					moveBuff2++;
				}

				/*Found a URL, write it down*/
				if(isOkayURL==1) {
					if(dmp->flags&DS_WRITE_LOCATION_STRING) {
						//Write the pointer where we found it
						sprintf(dmp->string_location, "Address: %p, ", (memlong) memInfo->BaseAddress + (moveBuff - buff));
						WriteFile(dmp->dumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);
					}
					WriteFile(dmp->dumpFile, moveBuff, moveBuff2 - moveBuff, (LPDWORD) &writtenBytes, NULL);
					WriteFile(dmp->dumpFile, "\r\n", strlen("\r\n"), (LPDWORD) &writtenBytes, NULL);

					moveBuff += moveBuff2 - moveBuff -1;
				}
				
			}
		}				
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}
}

void DestroyFilterGeneralASCII(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

/*-----------------------------------------------------End Filter ASCII-----------------------------------------------------*/

/*--------------------------------------------------------Filter UNICODE--------------------------------------------------------*/
DUMP_SESSION* InitiateFilterGeneralUNICODE(char* location, char* argument, int analyse_type) {
	return CreateUNICODEMemoryDump(location, "General_URL_Unicode_Search.txt", 0, 
		(argument==(char*) -1 ? DS_WRITE_LOCATION_STRING : strtol(argument, NULL, 10))
	);
}
/*Search for (UNICODE) URL's in the memory (starting with http://)*/
void ApplyFilterGeneralUNICODE(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	char* moveBuff=buff;
	wchar_t* moveBuff2=0;
	int writtenBytes;
	int readSize=0;
	int isOkayURL=0;

	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {
		for(; moveBuff<(buff + memInfo->RegionSize); moveBuff++) {
			//Is it possible a URL?
			if(memcmp(moveBuff, L"http://",  7*2) == 0) {
				moveBuff2 = (wchar_t*) moveBuff;
				moveBuff2 += 7*2; //skip 'http://'

				isOkayURL = 1;

				/*Check if the domain meets the requirments:
				- Letters a - z
				- numbers 0-9
				- '-' or '.' */
				while(*moveBuff2 != 0) {
					if((*moveBuff2 < 'a' || *moveBuff2 > 'z') 
						&& (*moveBuff2 < '0' || *moveBuff2 > '9') 
						&& *moveBuff2 != '-'
						&& *moveBuff2 != '.') {
							if(*moveBuff2 == '/') //domain stops, path begins
								break;
							isOkayURL = 0;
							break;
					}
					moveBuff2++;
				}

				/*Check if the URL meets the requirments:
				- No space (this is a %20)
				- no newline (0D of 0A)
				*/
				while(*moveBuff2 != 0) {
					if(*moveBuff2 == ' ' || *moveBuff2 == 0x0D || *moveBuff2 == 0x0A) {
						isOkayURL = 0;
						break;
					}
					moveBuff2++;

				}

				/*Found a URL, write it down*/
				if(isOkayURL==1) {
					if(dmp->flags&DS_WRITE_LOCATION_STRING) {
						//Write the pointer where we found it
						wsprintfW((LPWSTR) dmp->string_location, L"Address: %p, \0", (memlong) memInfo->BaseAddress + (moveBuff - buff));
						WriteFile(dmp->dumpFile, dmp->string_location, wcslen((wchar_t*) dmp->string_location) *2, (LPDWORD) &writtenBytes, NULL);
					}
					WriteFile(dmp->dumpFile, moveBuff, (char*) moveBuff2 -  moveBuff, (LPDWORD) &writtenBytes, NULL);
					WriteFile(dmp->dumpFile, L"\r\n", strlen("\r\n") * 2, (LPDWORD) &writtenBytes, NULL);

					moveBuff += (char*) moveBuff2 - moveBuff -1;
				}

				
			}
		}				
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}
}

void DestroyFilterGeneralUNICODE(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

/*-----------------------------------------------------End Filter UNICODE-----------------------------------------------------*/