/**
 * Memory Analyser  version 1.00
 * Copyright (c) 2018, Tycho Veltmeijer.
 * All rights reserved.
**/

#include "Filter_Safari.h"

/*Safari Filter conatins 3 filters. Two searching for URLS and one for Cookies, the filters are described in "5. Safari".
Filter 1: URL's, described in "Header 2"
Filter 2: URL's, described in "Header 3"
Filter 3: cookies, described in "5.2 Cookies" */

/*--------------------------------------------------------Filter 1--------------------------------------------------------*/
DUMP_SESSION* InitiateSafariFilter1(char* location, char* argument, int analyse_type) {
		return CreateMemoryDump(location, "Safari_URL_Search_Header2.txt", 0, 
		(argument==(char*) -1 ? DS_WRITE_LOCATION_STRING : strtol(argument, NULL, 10))
	);
}

/*Look for URL's in Safari for looking for some bytes where URL's seem to start with.*/
int ApplySafariFilter1(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	char signature[7]={0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}; //'Signature'
	char* moveBuff=buff;	//Cur pos
	char* moveBuff2;		//back-up cur pos
	int writtenBytes;		//Returnvalue from Writefile()
	int readSize=0;			//size of found URL
	int URLcheckPos=0;		//Check for overflow
	int isOkayURL;			//Boolean value, if try: it's an URL
	int foundDot;			//Boolean true: we found the 'dot'
	

	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {
		for(; moveBuff<(buff + memInfo->RegionSize); moveBuff++) {
			
			if(memcmp(moveBuff, signature, 7) == 0) { //check on signature
				moveBuff+=7; //skip signature
				readSize = (int) *(int*)moveBuff; //read out length
				moveBuff+=4+4; //skip to URL

				isOkayURL = 1;
				foundDot=0;
				moveBuff2= moveBuff;

				URLcheckPos = readSize;

				/*look if the domain meets it's requirments:
				- Letters a - z
				- numbers 0-9
				- '-' or '.' */
				while(*moveBuff2 != 0 && URLcheckPos!=0) {
					if((*moveBuff2 < 'a' || *moveBuff2 > 'z') 
					&& (*moveBuff2 < '0' || *moveBuff2 > '9') 
					&& *moveBuff2 != '-'
					&& *moveBuff2 != '.') {
						if(*moveBuff2 == ':' && *(moveBuff2+1) == '/' && *(moveBuff2+2) == '/') {
							//"://" is the end of the protocolname, next up is the domain
							moveBuff2+=2;
						} else {
							if(*moveBuff2 == '/') 
								break;
							isOkayURL = 0;
							break;
						}
					} else {
						if(*moveBuff2 == '.')
							foundDot=1;
					}

					moveBuff2++;
					URLcheckPos--;
				}

				/*Expecting a dot (like .nl / .com / .net)*/
				if(foundDot==0)
					isOkayURL = 0;

				/*look if the URL meets it's requirments:
				- no spase (space is replaced with %20)
				- no new line (0D of 0A)
				*/
				while(*moveBuff2 != 0 && URLcheckPos!=0) {
					if(*moveBuff2 == ' ' || *moveBuff2 == 0x0D || *moveBuff2 == 0x0A) {
						isOkayURL = 0;
						break;
					}
					moveBuff2++;
					URLcheckPos--;
				}

				if(isOkayURL && URLcheckPos==0) {
					/*write URL to file*/
					if(dmp->flags&DS_WRITE_LOCATION_STRING) {
						printf("Address: %p: ", (memlong) memInfo->BaseAddress + (moveBuff - buff));
						sprintf(dmp->string_location, "Address: %p: ", (memlong) memInfo->BaseAddress + (moveBuff - buff));
						WriteFile(dmp->dumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);
					}

					printf("%s\n", moveBuff);
					WriteFile(dmp->dumpFile, moveBuff, readSize, (LPDWORD) &writtenBytes, NULL);
					WriteFile(dmp->dumpFile, "\r\n", strlen("\r\n"), (LPDWORD) &writtenBytes, NULL);

					//Next...
					moveBuff += readSize -1;
				} else {
					moveBuff -= 7+4+4;
				}
			}
		}				
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}

	return 0;
}

void DestroySafariFilter1(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

/*-----------------------------------------------------End Filter 1-----------------------------------------------------*/

/*--------------------------------------------------------Filter 2--------------------------------------------------------*/
DUMP_SESSION* InitiateSafariFilter2(char* location, char* argument, int analyse_type) {
		return CreateUNICODEMemoryDump(location, "Safari_URL_Search_Header3.txt", 0, 
		(argument==(char*) -1 ? DS_WRITE_LOCATION_STRING : strtol(argument, NULL, 10))
	);
}

/*Look for URL's in Safari for looking for some bytes where URL's seem to start with.*/
int ApplySafariFilter2(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	/*Signature1 contains string "URL"*/
	char signature1[10]={0x00, 0x55, 0x00, 0x52, 0x00, 0x4C, 0x00, 0x00, 0x00, 0x10}; 
	/*Signature1 contains string "Link"*/
	char signature2[7] = {0x4C, 0x00, 0x69, 0x00, 0x6E, 0x00, 0x6B};


	char* moveBuff=buff;		//Cur pos
	wchar_t* moveBuff2;			//Back-up cur pos
	int writtenBytes;			//Returnvalue from Writefile()
	int readSize=0;				//size of the URL
	int readSizeinUnicode=0;	//ammount of characters of the URL


	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {
		for(; moveBuff<(buff + memInfo->RegionSize); moveBuff++) {
			if(memcmp(moveBuff, signature1, 10) == 0
				&& memcmp(moveBuff-7, signature2, 7) != 0) {
				
				moveBuff+=10+3; //skip to length of URL
				readSizeinUnicode = *(int*)moveBuff; //ammount of characters
				moveBuff+=4; 
				readSize = *(int*)moveBuff; //size of the string
				moveBuff+=4+4; //Zet de pointer op de string*/
				

				if(readSize/2 == readSizeinUnicode	&& readSize != 0) {  

						/*Write address and found URL*/
						if(dmp->flags&DS_WRITE_LOCATION_STRING) {
							printf("Address: %p: ",(memlong) memInfo->BaseAddress + (moveBuff - buff));
							wsprintfW((LPWSTR) dmp->string_location, L"Address: %p, \0", (memlong) memInfo->BaseAddress + (moveBuff - buff));
							WriteFile(dmp->dumpFile, dmp->string_location, wcslen((wchar_t*) dmp->string_location)*2, (LPDWORD) &writtenBytes, NULL);
						}

						wprintf(L"%s\r\n", moveBuff);
						WriteFile(dmp->dumpFile, moveBuff, readSize, (LPDWORD) &writtenBytes, NULL);
						WriteFile(dmp->dumpFile, L"\r\n", 4, (LPDWORD) &writtenBytes, NULL);


						//Next...
						moveBuff += readSize -1;
					//}

				} else {
					moveBuff -= 10+3+4+4+4;
				}
			}
		}				
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}

	return 0;
}

void DestroySafariFilter2(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

/*-----------------------------------------------------End Filter 2-----------------------------------------------------*/


/*--------------------------------------------------------Filter 3--------------------------------------------------------*/
DUMP_SESSION* InitiateSafariFilter3(char* location, char* argument, int analyse_type) {
		return CreateMemoryDump(location, "Safari_Cookie_Search.txt", 0, 
		(argument==(char*) -1 ? DS_WRITE_LOCATION_STRING : strtol(argument, NULL, 10))
	);
}

/*Find cookies*/
int ApplySafariFilter3(DUMP_SESSION* dmp, HANDLE process, MEMORY_BASIC_INFORMATION* memInfo, char* buff, SIZE_T overflowSize) {
	/*Signature...

		"Set-Cookie"

	...of a cookie
	*/
	char signature1[10]={0x53, 0x65, 0x74, 0x2D, 0x43, 0x6F, 0x6F, 0x6B, 0x69, 0x65};
	/*Afther this there are 6 random bytes*/

	/*And afther that, signature2*/
	char signature2[16]={0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	/*Next up 4 bytes with the size of the cookie*/
	/*next up 4 zero bytes*/
	char signature3[4]={0x00, 0x00, 0x00, 0x00};

	/*Signature4 is used to find cookies*/
	char signature4[7]={0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
	/*Next is size of string*/
	/*Behind the size of a string is the next combination of bytes:*/
	char signature5[8]={0x00, 0x00, 0x00, 0x00, 0x68, 0x74, 0x74, 0x70};

	char* moveBuff=buff; //cur pos
	char* moveBuff2;     //back-up cur pos
	int writtenBytes;	 //Returnvalue from Writefile()
	int readSize=0;		 //temp size
	int readSize_tmp;
	int readSize_tmp2;
	int url_size;
	int i;

	if(memInfo->State!=MEM_FREE && memInfo->State!=MEM_RESERVE) {
		for(; moveBuff<(buff + memInfo->RegionSize); moveBuff++) {
			readSize = * ((int*) (moveBuff + 10+6+16));

			if(	memcmp(moveBuff, signature1,  9) == 0
			&& memcmp(moveBuff+10+6, signature2,  16) == 0
			&& memcmp(moveBuff+10+6+16+4, signature3,  4) == 0
			) { //Look for "Set-Cookie" and the combination of other combination of bytes
				moveBuff2 = moveBuff;

				/*FInd URL that placed this cookie*/
				while(moveBuff2 != (buff+overflowSize)) {
					if(memcmp(moveBuff2, signature5, 8) == 0
					&& memcmp(moveBuff2-4-7, signature4, 7) == 0
					) {
						url_size = * ((int*) (moveBuff2 - 4));
						i = url_size;
						while(i!=0) {
							if(*(moveBuff2+i) == ' ')
								break;

							i--;
						}

						if(i==0) {
							moveBuff2 += 4;
							break;
						}
					}
					moveBuff2++;
				}

				//Is URL at the end of the memory.. then no coockie
				if(moveBuff2 == (buff+overflowSize))
					moveBuff2 = 0;
				
				/*Write address and URL to file*/
				if(dmp->flags&DS_WRITE_LOCATION_STRING) {
					printf("Address: %p: ", (memlong) memInfo->BaseAddress + (moveBuff - buff));
					sprintf(dmp->string_location, "Address: %p: \r\n", (memlong) memInfo->BaseAddress + (moveBuff - buff));
					WriteFile(dmp->dumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);
				}				

				if(moveBuff2) {
					WriteFile(dmp->dumpFile, "Geplaatst door: ", strlen("Geplaatst door: "), (LPDWORD) &writtenBytes, NULL);
					WriteFile(dmp->dumpFile, moveBuff2, url_size, (LPDWORD) &writtenBytes, NULL);
				}

				moveBuff += 10+6+16+4+4; //skip checked bytes

				
				readSize_tmp = readSize;

				/*Loop though all the cookies and write them to the file*/
				while(readSize_tmp!=0) {
					readSize_tmp2 = 0;

					if(*(moveBuff-1) == ',' && *(moveBuff-5) != '=' && *(moveBuff)==' ') {
						moveBuff++;
						readSize_tmp--;

						WriteFile(dmp->dumpFile, "\r\n", strlen("\r\n"), (LPDWORD) &writtenBytes, NULL);

						if(dmp->flags&DS_WRITE_LOCATION_STRING) {
							printf("Address: %p: \r\n", (memlong) memInfo->BaseAddress + (moveBuff - buff));
							sprintf(dmp->string_location, "\r\nAddress: %p: ", (memlong) memInfo->BaseAddress + (moveBuff - buff));
							WriteFile(dmp->dumpFile, dmp->string_location, strlen(dmp->string_location), (LPDWORD) &writtenBytes, NULL);
						}
						
						if(moveBuff2) {
							WriteFile(dmp->dumpFile, "\r\nGeplaatst door: ", strlen("\r\nGeplaatst door: "), (LPDWORD) &writtenBytes, NULL);
							WriteFile(dmp->dumpFile, moveBuff2, url_size, (LPDWORD) &writtenBytes, NULL);
						}
						
					}

					/*find properties of cookie*/
					while(*(moveBuff+readSize_tmp2)!=';' && 
						!(
							*(moveBuff+readSize_tmp2)==',' 
							&& *(moveBuff+readSize_tmp2-4) != '=' 
							&& *(moveBuff+readSize_tmp2+1)==' '
						) 
						&& readSize_tmp!=0) {
						readSize_tmp2++;
						readSize_tmp--;
					}

					/*write properties to file*/
					WriteFile(dmp->dumpFile, "\r\n", strlen("\r\n"), (LPDWORD) &writtenBytes, NULL);
					if(*moveBuff==' ')
						WriteFile(dmp->dumpFile, moveBuff+1, readSize_tmp2-1, (LPDWORD) &writtenBytes, NULL);
					else
						WriteFile(dmp->dumpFile, moveBuff, readSize_tmp2, (LPDWORD) &writtenBytes, NULL);

					moveBuff += readSize_tmp2+1;
					if(readSize_tmp) readSize_tmp--;
				}

				WriteFile(dmp->dumpFile, "\r\n\r\n", strlen("\r\n\r\n"), (LPDWORD) &writtenBytes, NULL);

				//Next...
				moveBuff -= 2;
			}
		}
		//}				
	} else {
		dmp->unlocated_size += memInfo->RegionSize;
	}

	return 0;
}

void DestroySafariFilter3(DUMP_SESSION* filter) {
	Destroy_MemoryDump(filter);
}

/*-----------------------------------------------------End Filter 3-----------------------------------------------------*/

